/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.team.ui.handlers;

import java.io.File;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;
import org.simantics.db.ChangeSetIdentifier;
import org.simantics.db.Session;
import org.simantics.db.common.utils.Logger;
import org.simantics.db.service.ManagementSupport;
import org.simantics.db.service.TeamSupport;
import org.simantics.team.ui.Command;
import org.simantics.ui.SimanticsUI;

public class DumpToSelectedRevisionHandler extends AbstractHandler {
    public DumpToSelectedRevisionHandler() {
    }
    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        ISelection sel = HandlerUtil.getCurrentSelection(event);
        if (sel instanceof IStructuredSelection) {
            IStructuredSelection ss = (IStructuredSelection)sel;
            try {
                Object o = ss.getFirstElement();
                if (null == o || !(o instanceof Command))
                    return null;
                Command command = (Command)o;
                ChangeSetIdentifier csi = command.getChangeSetIdentifier();
                if (null == csi)
                    return null;
                long csid = csi.getId();
                Session session = SimanticsUI.getSession();
                ManagementSupport ms = session.getService(ManagementSupport.class);
                ms.dumpRevision(csid);
                File wsFolder = new File(Platform.getInstanceLocation().getURL().getFile());
                File frFolder = new File(wsFolder, "db");
                File toFolder = new File(wsFolder, "Dump-ws");
                TeamSupport ts = session.getService(TeamSupport.class);
                ts.dump(frFolder, toFolder, csid);
                SynchroniseAllHandler.show(session, null, toFolder);
          } catch (Exception e) {
              Logger.defaultLogError(e);
          }
            
        }
        return null;
    }
}
